<?php

namespace Drupal\rc_opensocial\Plugin\Block;

use Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException;
use Drupal\Component\Plugin\Exception\PluginNotFoundException;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\rc\Services\RcUser;
use Drupal\Core\Block\BlockBase;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\rc_group\Services\RcGroup;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a 'RcOsGroupBlock' block.
 *
 * @Block(
 *  id = "rc_os_group_block",
 *  admin_label = @Translation("Chat Opensocial group block"),
 * )
 */
class RcOsGroupBlock extends BlockBase implements ContainerFactoryPluginInterface {

  /**
   * Current path.
   *
   * @var \Drupal\Component\Plugin\Context\ContextInterface[]|void
   */
  private $routeMatch;

  /**
   * Current user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  private AccountInterface $account;


  /**
   * Current group.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  private EntityTypeManagerInterface $entityTypeManager;

  /**
   * RcUser service initiated from container.
   *
   * @var \Drupal\rc\Services\RcUser
   */
  private RcUser $rcUser;

  /**
   * RcGroup service initiated from container.
   *
   * @var \Drupal\rc_group\Services\RcGroup
   */
  private RcGroup $rcGroup;

  /**
   * @param array $configuration
   * @param string $plugin_id
   * @param mixed $plugin_definition
   * @param \Drupal\Core\Session\AccountInterface $account
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   * @param \Drupal\rc\Services\RcUser $rcUser
   * @param \Drupal\rc_group\Services\RcGroup $rcGroup
   * @param $routeMatch
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, AccountInterface $account, EntityTypeManagerInterface $entityTypeManager, RcUser $rcUser, RcGroup $rcGroup, $routeMatch) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);

    $this->account = $account;
    $this->entityTypeManager = $entityTypeManager;
    $this->rcUser = $rcUser;
    $this->rcGroup = $rcGroup;
    $this->routeMatch = $routeMatch;
    $params = $this->routeMatch->getParameters();
    foreach ($params as $param) {
      if ($param instanceof EntityInterface) {
        $this->entity = $param;
      }
    }
  }

  /**
   * Pass service through container.
   *
   * @param \Symfony\Component\DependencyInjection\ContainerInterface $container
   * @param array $configuration
   * @param $plugin_id
   * @param $plugin_definition
   *
   * @return \Drupal\rc_opensocial\Plugin\Block\RcOsGroupBlock
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): RcOsGroupBlock {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('current_user'),
      $container->get('entity_type.manager'),
      $container->get('rc.user'),
      $container->get('rc_group.group'),
      $container->get('current_route_match')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheMaxAge(): int {
    return 0;
  }

  /**
   * {@inheritdoc}
   */
  public function build() {

    $build = [];
    $build['cache'] = ['max-age' => 0];

    $group = $this->entity;
    if (!$group) {
      return $build;
    }

    // Disable block if not on a group entity.
    if ($group->getEntityTypeId() != 'group') {
      return $build;
    }

    // Check if the group has a chat group.
    if (!$group->field_rc_room_id->value) {

      return $build;
    }

    // Check if the current user has a Rocket chat account and a valid personal
    // token.
    $userId = $this->account->id();

    try {
      $user = $this->entityTypeManager->getStorage('user')->load($userId);
    }
    catch (InvalidPluginDefinitionException | PluginNotFoundException $e) {
    }

    // Define RcUser service.
    $rcUser = $this->rcUser;

    if ($user->field_rcid->value) {

      // Get the resume token from field.
      $token = $user->field_rc_token->value;
      // If token does not exist, generate token.
      if (!$token) {
        $rcUser->reGeneratePersonalAccessToken($user);
        // If token still not exist, return.
        $token = $user->field_rc_token->value;
        if (!$token) {

          return $build;
        }
      }

      // Check if the current user is a member in the group.
      if (!$group->getMember($user)) {

        return $build;
      }

      $loginToken = $rcUser->getAuthTokenByName($user);

      // Get the server url.
      $serverUrl = $rcUser->url();

      // Load the Rc Groups class.
      $rcGroup = $this->rcGroup;

      // Get the Rocket Chat group path.
      $rcGroupInfo = $rcGroup->groupInfo($group);

      // Skip if unable to get group info.
      if (!$rcGroupInfo)  {
        return $build;
      }

      $roomUrl = '/group/' . $rcGroupInfo->getName();

      // Get the host name to be used with deep linking.
      $host = preg_replace("(^https?://)", "", $serverUrl);

      // Get the block configurations.
      $config = $this->getConfiguration();

      // Set the template to be used. the template will not be loaded until all
      // conditions is true.
      $build['#theme'] = 'rc_os_group_block';

      // Set group name.
      $build['#group'] = $group->label();
      $build['#button_text'] = $config['button_text'];
      $build['#url'] = $serverUrl . $roomUrl . ($config['embedded'] ? '?layout=embedded' : '');
      $build['#token'] = $loginToken;
      $build['#width'] = $config['width'];
      $build['#height'] = $config['height'];
      $build['#web'] = $config['visibility_web'] ? 'https://go.rocket.chat/room?host=' . $host . '&resumeToken=' . $token : NULL;
      $build['#mobile'] = $config['visibility_mobile'] ? 'rocketchat://room?host=' . $host . '&resumeToken=' . $token : NULL;
      $build['#open_new_window'] = $config['open_new_window'];
      $build['#new_window_width'] = $config['new_window_width'];
      $build['#new_window_height'] = $config['new_window_height'];

      return $build;

    }

    return $build;

  }

  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $form_state): array {
    $form = parent::blockForm($form, $form_state);

    $config = $this->getConfiguration();

    $form['#tree'] = TRUE;

    $form['button_text'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Chat button text'),
      '#description' => $this->t('Choose the text that will appear inside the chat button.'),
      '#maxlength' => 255,
      '#size' => 40,
      '#default_value' => $config['button_text'] ?? 'Open group chat',
    ];

    $form['block_options'] = [
      '#type' => 'details',
      '#title' => $this->t('Basic configuration'),
      '#open' => TRUE,
      '#attributes' => [
        'id' => 'rc-basic-config',
      ],
    ];

    $form['block_options']['width'] = [
      '#type' => 'textfield',
      '#title' => $this->t('iFrame width'),
      '#description' => $this->t('The width of the iFrame in pixel or percent "200px" or "100%".'),
      '#maxlength' => 255,
      '#size' => 40,
      '#default_value' => $config['width'] ?? '350px',
    ];

    $form['block_options']['height'] = [
      '#type' => 'textfield',
      '#title' => $this->t('iFrame height'),
      '#description' => $this->t('The height of the iFrame. in pixel or percent "200px" or "100%"'),
      '#maxlength' => 255,
      '#size' => 40,
      '#default_value' => $config['height'] ?? 'calc(100vh - 50px)',
    ];

    $form['block_options']['visibility_web'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('show a deep link to open chat app using "https://go.rocket.chat"'),
      '#default_value' => $config['visibility_web'] ?? FALSE,
    ];

    $form['block_options']['visibility_mobile'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('show a deep link to open chat app using "rocketchat://"'),
      '#description' => $this->t('If checked, it shows a direct link to open the chat mobile app.'),
      '#default_value' => $config['visibility_mobile'] ?? FALSE,
    ];

    $form['block_options']['embedded'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Show the embedded version of the chat inside the iFrame'),
      '#description' => $this->t('If checked, it will show the embedded version of the chat inside the iFrame.'),
      '#default_value' => $config['embedded'] ?? TRUE,
    ];

    $form['block_options']['rc_message'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Error message'),
      '#description' => $this->t('An error message to show when chat is not available.'),
      '#maxlength' => 255,
      '#size' => 64,
      '#default_value' => $config['rc_message'] ?? 'Couldn\'t login to chat. Edit and save your account to update information, then try again.',
    ];

    $form['window_options'] = [
      '#type' => 'details',
      '#title' => $this->t('New window options'),
      '#open' => TRUE,
    ];

    $form['window_options']['open_new_window'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Show a button to open the chat in a new window.'),
      '#description' => $this->t('If checked, it will show a button link to open the chat in a new window.'),
      '#default_value' => $config['open_new_window'] ?? FALSE,
      '#attributes' => [
        'id' => 'rc-window-options',
      ],
    ];

    $form['window_options']['new_window_width'] = [
      '#type' => 'textfield',
      '#title' => $this->t('New window width'),
      '#description' => $this->t('The width of the new window. Insert the number without px, vh...etc.'),
      '#maxlength' => 255,
      '#size' => 40,
      '#default_value' => $config['new_window_width'] ?? '400',
      '#states' => [
        'visible' => [
          '#rc-window-options' => ['checked' => TRUE],
        ],
      ],
    ];

    $form['window_options']['new_window_height'] = [
      '#type' => 'textfield',
      '#title' => $this->t('New window height'),
      '#description' => $this->t('The height of the new window. Insert the number without px, vh...etc.'),
      '#maxlength' => 255,
      '#size' => 40,
      '#default_value' => $config['height'] ?? '700',
      '#states' => [
        'visible' => [
          '#rc-window-options' => ['checked' => TRUE],
        ],
      ],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   *
   * @throws \Drupal\Core\Entity\EntityMalformedException
   */
  public function blockSubmit($form, FormStateInterface $form_state) {
    parent::blockSubmit($form, $form_state);

    $values = $form_state->getValues();

    $this->configuration['button_text'] = $values['button_text'];
    $this->configuration['url'] = $values['block_options']['url'];
    $this->configuration['server_url'] = $this->rcUser->url();
    $this->configuration['width'] = strip_tags($values['block_options']['width']);
    $this->configuration['height'] = strip_tags($values['block_options']['height']);
    $this->configuration['visibility_web'] = $values['block_options']['visibility_web'];
    $this->configuration['visibility_mobile'] = $values['block_options']['visibility_mobile'];
    $this->configuration['embedded'] = $values['block_options']['embedded'];
    $this->configuration['open_new_window'] = $values['window_options']['open_new_window'];
    $this->configuration['new_window_width'] = strip_tags($values['window_options']['new_window_width']);
    $this->configuration['new_window_height'] = strip_tags($values['window_options']['new_window_height']);
  }

}
