<?php

namespace Drupal\rc\Plugin\Block;

use Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException;
use Drupal\Component\Plugin\Exception\PluginNotFoundException;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\rc\Services\RcUser;
use Drupal\Core\Block\BlockBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\file\Entity\File;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a 'RcUserBlock' block.
 *
 * @Block(
 *  id = "rc_user_popup_block",
 *  admin_label = @Translation("Chat user popup block"),
 * )
 */
class RcUserPopupBlock extends BlockBase implements ContainerFactoryPluginInterface {

  /**
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected AccountInterface $account;

  /**
   * @var \Drupal\rc\Services\RcUser
   */
  protected RcUser $rcUser;

  /**
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected EntityTypeManagerInterface $entityTypeManager;

  /**
   * @param array $configuration
   * @param string $plugin_id
   * @param mixed $plugin_definition
   * @param \Drupal\Core\Session\AccountInterface $account
   * @param \Drupal\rc\Services\RcUser $rcUser
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   */
  public function __construct(array $configuration,
    $plugin_id,
    $plugin_definition,
    AccountInterface $account,
    RcUser $rcUser,
    EntityTypeManagerInterface $entityTypeManager) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->account = $account;
    $this->rcUser = $rcUser;
    $this->entityTypeManager = $entityTypeManager;
  }

  /**
   * @param \Symfony\Component\DependencyInjection\ContainerInterface $container
   * @param array $configuration
   * @param $plugin_id
   * @param $plugin_definition
   *
   * @return static
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): RcUserPopupBlock {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('current_user'),
      $container->get('rc.user'),
      $container->get('entity_type.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheMaxAge() {
    return 0;
  }

  /**
   * {@inheritdoc}
   */
  public function build() {

    $build = [];

    // Getting the current user object.
    $userId = $this->account->id();

    try {
      $user = $this->entityTypeManager->getStorage('user')->load($userId);
    }
    catch (InvalidPluginDefinitionException | PluginNotFoundException $e) {
    }

    // Define RcUser service.
    $rcUser = $this->rcUser;

    // Check if user has a rocket chat ID field.
    if ($user && $user->field_rcid->value) {

      // Get resume token from field.
      $config = $this->getConfiguration();

      // Get the personal auth token.
      $token = $user->field_rc_token->value;

      // Generate token if it does not exist.
      if (!$token) {

        $rcUser->reGeneratePersonalAccessToken($user);
        $token = $user->field_rc_token->value;

        // If still no token, return.
        if (!$token) {
          return $build;
        }
      }

      // Get the authToken.
      $authToken = $rcUser->getAuthTokenByName($user);

      // Get the server URL.
      $serverUrl = $rcUser->url();

      $host = preg_replace("(^https?://)", "", $serverUrl);

      $build['#theme'] = 'rc_user_popup_block';
      // $build['#cache']["max-age"] = 0;
      $build['#url'] = $serverUrl . $config['url'] . ($config['embedded'] ? '?layout=embedded' : '');
      $build['#button_element'] = $config['button_element'];
      $build['#button_icon'] = $config['button_icon'];
      $build['#button_icon_path'] = $config['button_icon_path'];
      $build['#button_text'] = $config['button_text'];
      $build['#token'] = $authToken;
      $build['#width'] = $config['width'];
      $build['#height'] = $config['height'];
      $build['#web'] = $config['visibility_web'] ? 'https://go.rocket.chat/auth?host=' . $host . '&resumeToken=' . $token : NULL;
      $build['#mobile'] = $config['visibility_mobile'] ? 'rocketchat://auth?host=' . $host . '&resumeToken=' . $token : NULL;
      $build['#open_new_window'] = $config['open_new_window'];
      $build['#new_window_width'] = $config['new_window_width'];
      $build['#new_window_height'] = $config['new_window_height'];

      return $build;

    }

    return $build;

  }

  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $form_state) {
    $form = parent::blockForm($form, $form_state);

    $config = $this->getConfiguration();

    // Getting Rocket Chat config to generate server URL.
    $rcConfig = \Drupal::service('rc.user');
    $serverUrl = $rcConfig->url();

    $form['#tree'] = TRUE;

    $form['button_options'] = [
      '#type' => 'details',
      '#title' => t('Icon image and text'),
      '#open' => TRUE,
    ];

    $form['button_options']['button_element'] = [
      '#type' => 'select',
      '#title' => t('Select the button element'),
      '#options' => [
        'icon' => t('Use image icon'),
        'text' => t('Use text'),
      ],
      '#default_value' => $config['button_element'] ?? 'text',
      '#attributes' => [
        'id' => 'rc-button-config',
      ],
    ];

    $form['button_options']['icon_container'] = [
      '#type' => 'container',
      '#states' => [
        'visible' => [
          ':input[id="rc-button-config"]' => ['value' => 'icon'],
        ],
        'required' => [
          ':input[id="rc-button-config"]' => ['value' => 'icon'],
        ],
      ],
    ];

    $form['button_options']['icon_container']['button_icon'] = [
      '#type' => 'managed_file',
      '#title' => t('Icon button'),
      '#name' => 'button_icon',
      '#default_value' => $config['button_icon'],
      '#upload_location' => 'public://rc_chat_icons',
      '#upload_validators' => [
        'file_validate_is_image' => [],
        'file_validate_extensions' => ['gif png jpg jpeg'],
      ],
      '#states' => [
        'required' => [
          ':input[id="rc-button-config"]' => ['value' => 'icon'],
        ],
      ],
    ];

    $form['button_options']['button_text'] = [
      '#type' => 'textfield',
      '#title' => t('text Button'),
      '#maxlength' => 255,
      '#size' => 64,
      '#default_value' => $config['button_text'] ?? 'Chat',
      '#states' => [
        'visible' => [
          ':input[id="rc-button-config"]' => ['value' => 'text'],
        ],
        'required' => [
          ':input[id="rc-button-config"]' => ['value' => 'text'],
        ],
      ],
    ];

    $form['block_options'] = [
      '#type' => 'details',
      '#title' => t('Basic configuration'),
      '#open' => TRUE,
      '#attributes' => [
        'id' => 'rc-basic-config',
      ],
    ];

    $form['block_options']['url'] = [
      '#type' => 'textfield',
      '#title' => t('iFrame URL'),
      '#description' => t('The inner URL to be imbedded in as an iFrame.'),
      '#maxlength' => 255,
      '#size' => 64,
      '#default_value' => $config['url'],
      '#field_prefix' => $serverUrl,
    ];

    $form['block_options']['width'] = [
      '#type' => 'textfield',
      '#title' => t('iFrame width'),
      '#description' => t('The width of the iFrame in pixel or percent "200px" or "100%".'),
      '#maxlength' => 255,
      '#size' => 40,
      '#default_value' => $config['width'] ?? '100%',
    ];

    $form['block_options']['height'] = [
      '#type' => 'textfield',
      '#title' => t('iFrame height'),
      '#description' => t('The height of the iFrame. in pixel or percent "200px" or "100%"'),
      '#maxlength' => 255,
      '#size' => 40,
      '#default_value' => $config['height'] ?? '100vh',
    ];

    $form['block_options']['visibility_web'] = [
      '#type' => 'checkbox',
      '#title' => t('show a deep link to open chat app using "https://go.rocket.chat"'),
      '#description' => t('If checked, it shows a direct link to the chat website.'),
      '#default_value' => $config['visibility_web'] ?? FALSE,
    ];

    $form['block_options']['visibility_mobile'] = [
      '#type' => 'checkbox',
      '#title' => t('show a deep link to open chat app using "rocketchat://"'),
      '#description' => t('If checked, it shows a direct link to open the chat mobile app.'),
      '#default_value' => $config['visibility_mobile'] ?? FALSE,
    ];

    $form['block_options']['embedded'] = [
      '#type' => 'checkbox',
      '#title' => t('Show the embedded version of the chat inside the iFrame'),
      '#description' => t('If checked, it will show the embedded version of the chat inside the iFrame.'),
      '#default_value' => $config['embedded'] ?? TRUE,
    ];

    $form['window_options'] = [
      '#type' => 'details',
      '#title' => t('New window options'),
      '#open' => TRUE,
    ];

    $form['window_options']['open_new_window'] = [
      '#type' => 'checkbox',
      '#title' => t('Show a button to open the chat in a new window.'),
      '#description' => t('If checked, it will show a button link to open the chat in a new window.'),
      '#default_value' => $config['open_new_window'] ?? FALSE,
      '#attributes' => [
        'id' => 'rc-window-options',
      ],
    ];

    $form['window_options']['new_window_width'] = [
      '#type' => 'textfield',
      '#title' => t('New window width'),
      '#description' => t('The width of the new window. Insert the number without px, vh...etc.'),
      '#maxlength' => 255,
      '#size' => 40,
      '#default_value' => $config['new_window_width'] ?? '400',
      '#states' => [
        'visible' => [
          '#rc-window-options' => ['checked' => TRUE],
        ],
      ],
    ];

    $form['window_options']['new_window_height'] = [
      '#type' => 'textfield',
      '#title' => t('New window height'),
      '#description' => t('The height of the new window. Insert the number without px, vh...etc.'),
      '#maxlength' => 255,
      '#size' => 40,
      '#default_value' => $config['height'] ?? '700',
      '#states' => [
        'visible' => [
          '#rc-window-options' => ['checked' => TRUE],
        ],
      ],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   *
   * @throws \Drupal\Core\Entity\EntityMalformedException
   */
  public function blockSubmit($form, FormStateInterface $form_state) {
    parent::blockSubmit($form, $form_state);

    $values = $form_state->getValues();

    // Generating the icon image URL to be saved in the block config.
    $iconId = $values['button_options']['icon_container']['button_icon'][0];
    $iconUrl = NULL;
    if (isset($iconId)) {
      $iconUrl = File::load($iconId)->createFileUrl();
    }

    $this->configuration['url'] = $values['block_options']['url'];
    $this->configuration['button_element'] = $values['button_options']['button_element'];
    $this->configuration['button_text'] = strip_tags($values['button_options']['button_text']);
    $this->configuration['button_icon'] = $values['button_options']['icon_container']['button_icon'];
    $this->configuration['button_icon_path'] = $iconUrl;
    $this->configuration['server_url'] = $this->rcUser->url();
    $this->configuration['width'] = strip_tags($values['block_options']['width']);
    $this->configuration['height'] = strip_tags($values['block_options']['height']);
    $this->configuration['visibility_web'] = $values['block_options']['visibility_web'];
    $this->configuration['visibility_mobile'] = $values['block_options']['visibility_mobile'];
    $this->configuration['embedded'] = $values['block_options']['embedded'];

    $this->configuration['open_new_window'] = $values['window_options']['open_new_window'];
    $this->configuration['new_window_width'] = strip_tags($values['window_options']['new_window_width']);
    $this->configuration['new_window_height'] = strip_tags($values['window_options']['new_window_height']);
  }

}
